@@ -470,18 +470,76 @@ class XNGram
 	}
 };
 
+static long fts_backend_xapian_memory_used() // KB
+{
+	FILE* file = fopen("/proc/self/status", "r");
+	int i;
+	char line[129];
+	const char* p;
+
+	if(file != NULL)
+	{
+		while (fgets(line, 128, file) != NULL)
+		{
+			if (strncmp(line, "VmSize:", 7) == 0)
+			{
+				i = strlen(line);
+				p = line+7;
+				while (*p <'0' || *p > '9') p++;
+				line[i-3] = '\0';
+				fclose(file);
+				return atol(p);
+			}
+		}	
+		fclose(file);
+	}
+	return 0;
+}
+
+static long fts_backend_xapian_memory_free() // KB
+{
+        FILE* file = fopen("/proc/meminfo", "r");
+        int i;
+        char line[129];
+        const char* p;
+
+        if(file != NULL)
+        {
+                while (fgets(line, 128, file) != NULL)
+                {
+                        if (strncmp(line, "MemFree:", 8) == 0)
+                        {
+                                i = strlen(line);
+                                p = line+8;
+                                while (*p <'0' || *p > '9') p++;
+                                line[i-3] = '\0';
+                                fclose(file);
+                                return atol(p);
+                        }
+                }
+                fclose(file);
+        }
+        return 0;
+}
+
 static bool fts_backend_xapian_test_memory()
 {
-	void* p = malloc(XAPIAN_COMMIT_MEMORY * 1024);
-	if(p == NULL)
+	rlim_t limit;
+
+        restrict_get_process_size(&limit);
+        long m = long((long)limit / 1024.0); // vsz_limit of dovecot.conf
+	long used = fts_backend_xapian_memory_used();
+	long fri = fts_backend_xapian_memory_free(); // Free RAM
+
+	if(m<1) 
 	{
-		return false;
-	}
-	else
-	{
-		free(p);
-		return true;
+		if(verbose>0) i_info("FTS Xapian: Memory stats : Used = %ld MB, Free = %ld MB",long(used/1024),long(fri/1024));
+		return (fri>used/2);
 	}
+
+	if(verbose>0) i_info("FTS Xapian: Memory stats : Used = %ld MB (%ld%%), Limit = %ld MB, Free = %ld MB",long(used/1024),long(used*100.0/m),long(m/1024),long(fri/1024));
+
+	return ((m>used*3.0/2)&&(fri>used/2));
 }
 
 static bool fts_backend_xapian_open_readonly(struct xapian_fts_backend *backend, Xapian::Database ** dbr)
@@ -497,7 +555,7 @@ static bool fts_backend_xapian_open_readonly(struct xapian_fts_backend *backend,
 	try
 	{
 		if(verbose>1) i_info("FTS Xapian: Opening DB (RO) %s",backend->db);
-		*dbr = new Xapian::Database(backend->db,Xapian::DB_CREATE_OR_OPEN);
+		*dbr = new Xapian::Database(backend->db,Xapian::DB_OPEN);
 	}
 	catch(Xapian::Error e)
 	{
@@ -522,7 +580,7 @@ static bool fts_backend_xapian_check_access(struct xapian_fts_backend *backend)
 	try
 	{
 		if(verbose>0) i_info("FTS Xapian: Opening DB (RW) %s",backend->db);
-		backend->dbw = new Xapian::WritableDatabase(backend->db,Xapian::DB_CREATE_OR_OPEN | Xapian::DB_RETRY_LOCK);
+		backend->dbw = new Xapian::WritableDatabase(backend->db,Xapian::DB_CREATE_OR_OPEN | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS);
 	}
 	catch(Xapian::Error e)
 	{
@@ -577,7 +635,6 @@ static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const
                 backend->dbw = NULL;
 		backend->commit_updates = 0;
 		backend->commit_time = commit_time;
-		backend->memory = 0;
 	}
 
 	if(verbose>0)
@@ -770,8 +827,26 @@ static int fts_backend_xapian_set_box(struct xapian_fts_backend *backend, struct
 	backend->guid = i_strdup(mb);
 	backend->boxname = i_strdup(box->name);
 	backend->db = i_strdup_printf("%s/db_%s",backend->path,mb);
-	backend->memory = 0;
-	
+
+	char * t = i_strdup_printf("%s/termlist.glass",backend->db);
+	struct stat sb;
+	if(!( (stat(t, &sb)==0) && S_ISREG(sb.st_mode)))
+	{
+		i_info("FTS Xapian: '%s' (%s) indexes do not exist. Initializing DB",backend->boxname,backend->db);
+		try
+		{
+			Xapian::WritableDatabase * db = new Xapian::WritableDatabase(backend->db,Xapian::DB_CREATE_OR_OVERWRITE | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS);
+			db->close();
+			delete(db);
+		}
+		catch(Xapian::Error e)
+		{
+			i_error("FTS Xapian: Can't create Xapian DB (%s) %s : %s - %s",backend->boxname,backend->db,e.get_type(),e.get_error_string());
+        	}
+	}
+	i_free(t);
+
+
         /* Performance calculator*/
         backend->perf_dt = current_time;
         backend->perf_uid=0;
@@ -916,8 +991,6 @@ bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid,
 		i_info("FTS Xapian: Ngram(%s) -> %ld items (total %ld KB)",h,ngram->size, ngram->memory/1024); 
 	}
 
-	backend->memory = backend->memory + ngram->memory;
-	
 	for(i=0;i<ngram->size;i++)
 	{
 		u = i_strdup_printf("%s%s",h,ngram->data[i]);
@@ -934,7 +1007,18 @@ bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid,
 	}
 	delete(ngram);
 
-	if(ok) dbx->replace_document(docid,*doc);
+	if(ok)
+	{
+		try
+		{
+			dbx->replace_document(docid,*doc);
+		}
+		catch (std::bad_alloc& ba)
+		{
+			i_error("FTS Xapian: Memory error '%s'",ba.what());
+			ok = false;
+		}
+	}
 
 	delete(doc);
 
@@ -992,10 +1076,10 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 	if(!ok) return false;
 
 	Xapian::Document * doc2 = new Xapian::Document();
-	Xapian::TermGenerator termgenerator;
+	Xapian::TermGenerator * termgenerator = new Xapian::TermGenerator();;
 	Xapian::Stem stem("en");
-	termgenerator.set_stemmer(stem);
-	termgenerator.set_document(*doc2);
+	termgenerator->set_stemmer(stem);
+	termgenerator->set_document(*doc2);
 
 	const char * h;
 	if(strcmp(field,"subject")==0) 
@@ -1008,30 +1092,28 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 	}
 	std::string s;
 	data->toUTF8String(s);
-	termgenerator.set_stemming_strategy(Xapian::TermGenerator::STEM_ALL);
-	termgenerator.index_text(s, 1, h);
+	termgenerator->set_stemming_strategy(Xapian::TermGenerator::STEM_ALL);
+	termgenerator->index_text(s, 1, h);
 	
 	long l = strlen(h);
 	long n = doc2->termlist_count();
-	Xapian::TermIterator ti = doc2->termlist_begin();
+	Xapian::TermIterator * ti = new Xapian::TermIterator(doc2->termlist_begin());
 
 	XNGram * ngram = new XNGram(p,f,h);
 	const char * c;
 	while(n>0)
 	{
-		s = *ti;
+		s = *(*ti);
 		c=s.c_str();	
 		if(strncmp(c,h,l)==0)
 		{
 			ngram->add(c+l);
 		}
-		ti++;
+		(*ti)++;
 		n--;
 	}
 
 	if(verbose>0) i_info("FTS Xapian: NGRAM(%s,%s) -> %ld items, max length=%ld, (total %ld KB)",field,h,ngram->size,ngram->maxlength,ngram->memory/1024);
-	backend->memory = backend->memory + ngram->memory;
-	delete(doc2);
 
 	char *t = (char*)i_malloc(sizeof(char)*(ngram->maxlength+6));
 	for(n=0;n<ngram->size;n++)
@@ -1049,8 +1131,22 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 	}
 	i_free(t);
 	delete(ngram);
+	delete(ti);
+	delete(termgenerator);
+	delete(doc2);
 
-	if(ok) dbx->replace_document(docid,*doc);
+	if(ok) 
+	{
+		try
+		{
+			dbx->replace_document(docid,*doc);
+		}
+		catch (std::bad_alloc& ba)
+		{
+			i_error("FTS Xapian: Memory error '%s'",ba.what());
+			ok = false;
+		}
+	}	
 
 	delete(doc);
 
