@@ -10,10 +10,9 @@ extern "C" {
 #include <sys/time.h>
 
 #define XAPIAN_FILE_PREFIX "xapian-indexes"
-#define XAPIAN_TERM_SIZELIMIT 245
-#define XAPIAN_COMMIT_ENTRIES 1000000
-#define XAPIAN_COMMIT_TIMEOUT 300
-#define XAPIAN_COMMIT_MEMORY 10240
+#define XAPIAN_TERM_SIZELIMIT 245L
+#define XAPIAN_COMMIT_ENTRIES 1000000L
+#define XAPIAN_COMMIT_TIMEOUT 300L
 #define XAPIAN_WILDCARD "wldcrd"
 #define XAPIAN_EXPUNGE_SIZE 3
 #define XAPIAN_EXPUNGE_HEADER 9
@@ -44,8 +43,6 @@ struct xapian_fts_backend
 	long commit_updates;
 	long commit_time;
 
-	long memory;
-
 	long perf_pt;
 	long perf_nb;
 	long perf_uid;
@@ -173,7 +170,7 @@ static void fts_backend_xapian_deinit(struct fts_backend *_backend)
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *)_backend;
 
-	if(verbose>0) i_info("FTS Xapian: Deinit %s",backend->path);
+	if(verbose>0) i_info("FTS Xapian: Deinit %s)",backend->path);
 
 	if(backend->guid != NULL) fts_backend_xapian_unset_box(backend);
 
@@ -506,6 +503,20 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 		i_error("FTS Xapian: Buildmore: Can not open db");
 		return -1;
 	}
+	
+	struct timeval tp;
+	gettimeofday(&tp, NULL);
+	long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
+	
+	if(!fts_backend_xapian_test_memory())
+	{
+		fts_backend_xapian_release(backend,"Low memory indexing", current_time);
+		if(!fts_backend_xapian_check_access(backend))
+		{
+			i_error("FTS Xapian: Buildmore: Can not open db (2)");
+			return -1;
+		}
+	}
 
 	bool ok=true;
 
@@ -520,13 +531,12 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 
 	backend->commit_updates++;
 
-	struct timeval tp;
 	gettimeofday(&tp, NULL);
-	long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
+	current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
 
-	if( (backend->commit_updates>XAPIAN_COMMIT_ENTRIES) || ((current_time - backend->commit_time) > XAPIAN_COMMIT_TIMEOUT*1000) || (!fts_backend_xapian_test_memory()))
+	if( (!ok) || (backend->commit_updates>XAPIAN_COMMIT_ENTRIES) || ((current_time - backend->commit_time) > XAPIAN_COMMIT_TIMEOUT*1000) )
 	{
-		if(verbose>0) i_info("FTS Xapian: Refreshing after %ld ms (vs %ld) and %ld updates (vs %ld) and %ld KB ...", current_time - backend->commit_time, XAPIAN_COMMIT_TIMEOUT*1000, backend->commit_updates, XAPIAN_COMMIT_ENTRIES, backend->memory/1024);
+		if(verbose>0) i_info("FTS Xapian: Refreshing after %ld ms (vs %ld) and %ld updates (vs %ld) ...", current_time - backend->commit_time, XAPIAN_COMMIT_TIMEOUT*1000, backend->commit_updates, XAPIAN_COMMIT_ENTRIES);
 		fts_backend_xapian_release(backend,"refreshing", current_time);
 	}
     	
